summaryrefslogtreecommitdiff
diff options
context:
space:
mode:
authorLogan Chien <tzuhsiang.chien@gmail.com>2013-12-05 00:34:11 +0000
committerLogan Chien <tzuhsiang.chien@gmail.com>2013-12-05 00:34:11 +0000
commit175fe6839353fe3659adba11c1f2b11d3451e7dc (patch)
tree43e8cada1481e8473343968088b52275218c046b
parentfaf4d59137e85f917f868e784c8d83ccc29c4b7f (diff)
downloadllvm-175fe6839353fe3659adba11c1f2b11d3451e7dc.tar.gz
llvm-175fe6839353fe3659adba11c1f2b11d3451e7dc.tar.bz2
llvm-175fe6839353fe3659adba11c1f2b11d3451e7dc.tar.xz
[mc] Fix ELF st_other flag.
ELF_Other_Weakref and ELF_Other_ThumbFunc seems to be LLVM internal ELF symbol flags. These should not be emitted to object file. This commit defines ELF_STO_Shift for the target-defined flags for st_other, and increase the value of ELF_Other_Shift to 16. git-svn-id: https://llvm.org/svn/llvm-project/llvm/trunk@196440 91177308-0d34-0410-b5e6-96231b3b80d8
-rw-r--r--include/llvm/MC/MCELFSymbolFlags.h10
-rw-r--r--lib/MC/ELFObjectWriter.cpp3
-rw-r--r--lib/MC/MCELF.cpp6
-rw-r--r--test/MC/ELF/thumb-st_other.s19
4 files changed, 29 insertions, 9 deletions
diff --git a/lib/MC/ELFObjectWriter.cpp b/lib/MC/ELFObjectWriter.cpp
index 9899bb2eac..972c64cc56 100644
--- a/lib/MC/ELFObjectWriter.cpp
+++ b/lib/MC/ELFObjectWriter.cpp
@@ -551,8 +551,7 @@ void ELFObjectWriter::WriteSymbol(MCDataFragment *SymtabF,
// Other and Visibility share the same byte with Visibility using the lower
// 2 bits
uint8_t Visibility = MCELF::GetVisibility(OrigData);
- uint8_t Other = MCELF::getOther(OrigData) <<
- (ELF_Other_Shift - ELF_STV_Shift);
+ uint8_t Other = MCELF::getOther(OrigData) << (ELF_STO_Shift - ELF_STV_Shift);
Other |= Visibility;
uint64_t Value = SymbolValue(Data, Layout);
diff --git a/lib/MC/MCELF.cpp b/lib/MC/MCELF.cpp
index ebb189e543..0a9cd31dda 100644
--- a/lib/MC/MCELF.cpp
+++ b/lib/MC/MCELF.cpp
@@ -72,13 +72,13 @@ unsigned MCELF::GetVisibility(MCSymbolData &SD) {
// Other is stored in the last six bits of st_other
// st_other values are stored in the second byte of get/setFlags
void MCELF::setOther(MCSymbolData &SD, unsigned Other) {
- uint32_t OtherFlags = SD.getFlags() & ~(0x3f << ELF_Other_Shift);
- SD.setFlags(OtherFlags | (Other << ELF_Other_Shift));
+ uint32_t OtherFlags = SD.getFlags() & ~(0x3f << ELF_STO_Shift);
+ SD.setFlags(OtherFlags | (Other << ELF_STO_Shift));
}
unsigned MCELF::getOther(MCSymbolData &SD) {
unsigned Other =
- (SD.getFlags() & (0x3f << ELF_Other_Shift)) >> ELF_Other_Shift;
+ (SD.getFlags() & (0x3f << ELF_STO_Shift)) >> ELF_STO_Shift;
return Other;
}
diff --git a/test/MC/ELF/thumb-st_other.s b/test/MC/ELF/thumb-st_other.s
new file mode 100644
index 0000000000..8750c2bba5
--- /dev/null
+++ b/test/MC/ELF/thumb-st_other.s
@@ -0,0 +1,19 @@
+@ Check the value of st_other for thumb function.
+
+@ ARM does not define any st_other flags for thumb function. The value
+@ for st_other should always be 0.
+
+@ RUN: llvm-mc < %s -triple thumbv5-linux-gnueabi -filetype=obj -o - \
+@ RUN: | llvm-readobj -t | FileCheck %s
+
+ .syntax unified
+ .text
+ .align 2
+ .thumb_func
+ .global main
+ .type main,%function
+main:
+ bx lr
+
+@ CHECK: Name: main
+@ CHECK: Other: 0
diff --git a/include/llvm/MC/MCELFSymbolFlags.h b/include/llvm/MC/MCELFSymbolFlags.h
index d0e1dace99..782e7d6126 100644
--- a/include/llvm/MC/MCELFSymbolFlags.h
+++ b/include/llvm/MC/MCELFSymbolFlags.h
@@ -21,10 +21,12 @@
namespace llvm {
enum {
- ELF_STT_Shift = 0, // Shift value for STT_* flags.
- ELF_STB_Shift = 4, // Shift value for STB_* flags.
- ELF_STV_Shift = 8, // Shift value for STV_* flags.
- ELF_Other_Shift = 10 // Shift value for other flags.
+ ELF_STT_Shift = 0, // Shift value for STT_* flags.
+ ELF_STB_Shift = 4, // Shift value for STB_* flags.
+ ELF_STV_Shift = 8, // Shift value for STV_* flags.
+ ELF_STO_Shift = 10, // Shift value for STO_* flags.
+ ELF_Other_Shift = 16 // Shift value for llvm local flags,
+ // not part of the final object file
};
enum ELFSymbolFlags {